# This file mainly exists to allow python setup.py test to work.
# flake8: noqa
import os
import sys

os.environ['DJANGO_SETTINGS_MODULE'] = 'tests.settings'
test_dir = os.path.join(os.path.dirname(__file__), 'tests')
sys.path.insert(0, test_dir)

import django
from django.test.utils import get_runner
from django.conf import settings


def runtests():
    # Disable migrations for our app during tests to avoid requiring migration files
    existing_migration_modules = getattr(settings, 'MIGRATION_MODULES', {}) or {}
    settings.MIGRATION_MODULES = {**existing_migration_modules, 'jwt_allauth': None}
    TestRunner = get_runner(settings)
    test_runner = TestRunner(verbosity=1, interactive=True)
    if hasattr(django, 'setup'):
        django.setup()
    failures = test_runner.run_tests(['tests'])
    sys.exit(bool(failures))


if __name__ == '__main__':
    runtests()



from rest_framework import serializers

from jwt_allauth.tokens.models import RefreshTokenWhitelistModel, GenericTokenModel


class RefreshTokenWhitelistSerializer(serializers.ModelSerializer):
    """
    User model w/o password
    """

    class Meta:
        model = RefreshTokenWhitelistModel
        exclude = ('id',)


class GenericTokenModelSerializer(serializers.ModelSerializer):
    """
    User model w/o password
    """

    class Meta:
        model = GenericTokenModel
        exclude = ('id',)



from django.conf import settings
from django.urls import path
from django.views.generic import TemplateView

from jwt_allauth.constants import EMAIL_VERIFIED_REDIRECT
from jwt_allauth.registration.email_verification.views import VerifyEmailView
from jwt_allauth.registration.views import RegisterView
from jwt_allauth.utils import get_template_path

urlpatterns = [
    path('', RegisterView.as_view(), name='rest_register'),
]

if getattr(settings, 'EMAIL_VERIFICATION', False):
    urlpatterns.extend([
        path('verification/<str:key>/', VerifyEmailView.as_view(), name='account_confirm_email'),

        # This url is used by django-allauth and empty TemplateView is
        # defined just to allow reverse() call inside app, for example when email
        # with verification link is being sent, then it's required to render email
        # content.

        # account_confirm_email - You should override this view to handle it in
        # your API client somehow and then, send post to /verify-email/ endpoint
        # with proper key.
        # If you don't want to use API on that step, then just use ConfirmEmailView
        # view from:
        # django-allauth https://github.com/pennersr/django-allauth/blob/master/allauth/account/views.py
        path('account_email_verification_sent/', TemplateView.as_view(), name='account_email_verification_sent'),
    ])

    if getattr(settings, EMAIL_VERIFIED_REDIRECT, None) is None:
        urlpatterns.append(
            path('verified/', TemplateView.as_view(
                template_name='email/verified.html'), name='jwt_allauth_email_verified'),
        )



from datetime import timedelta
from importlib import reload

import allauth.app_settings
import rest_framework_simplejwt.settings
from django.apps import AppConfig


class JWTAllauthAppConfig(AppConfig):
    name = 'jwt_allauth'
    verbose_name = "JWT Allauth"
    default_auto_field = 'django.db.models.BigAutoField'

    def ready(self):
        from django.conf import settings

        if not getattr(settings, 'ROTATE_REFRESH_TOKENS', True):
            raise ValueError('Refresh token rotation is compulsory.')
        if getattr(settings, 'BLACKLIST_AFTER_ROTATION', False):
            raise ValueError('Token blacklist is not supported.')

        settings.EMAIL_VERIFICATION = getattr(settings, 'EMAIL_VERIFICATION', False)
        if not hasattr(settings, 'ACCOUNT_ADAPTER'):
            settings.ACCOUNT_ADAPTER = 'jwt_allauth.adapter.JWTAllAuthAdapter'
        if hasattr(settings, 'ACCOUNT_LOGIN_METHODS') and settings.ACCOUNT_LOGIN_METHODS != {'email'}:
            raise ValueError('Only login email is supported.')
        settings.ACCOUNT_LOGIN_METHODS = {'email'}
        if (
                hasattr(settings, 'ACCOUNT_SIGNUP_FIELDS') and
                sorted(settings.ACCOUNT_SIGNUP_FIELDS) != ['email*', 'password1*', 'password2*']
        ):
            raise ValueError('Only login email is supported.')
        settings.ACCOUNT_SIGNUP_FIELDS = ['email*', 'password1*', 'password2*']

        if not hasattr(settings, 'SITE_ID'):
            settings.SITE_ID = 1
        if not hasattr(settings, 'ACCOUNT_EMAIL_VERIFICATION'):
            settings.ACCOUNT_EMAIL_VERIFICATION = 'mandatory' if settings.EMAIL_VERIFICATION else 'none'
        if not hasattr(settings, 'UNIQUE_EMAIL'):
            settings.UNIQUE_EMAIL = True
        if not hasattr(settings, 'ACCOUNT_EMAIL_SUBJECT_PREFIX'):
            settings.ACCOUNT_EMAIL_SUBJECT_PREFIX = ''

        simple_jwt_settings = {
            "BLACKLIST_AFTER_ROTATION": False,
            "UPDATE_LAST_LOGIN": True,

            "ALGORITHM": "HS256",
            "SIGNING_KEY": getattr(settings, 'JWT_SECRET_KEY', settings.SECRET_KEY),
            "VERIFYING_KEY": "",
            "AUDIENCE": None,
            "ISSUER": None,
            "JSON_ENCODER": None,
            "JWK_URL": None,
            "LEEWAY": 0,

            "AUTH_HEADER_TYPES": ("Bearer",),
            "AUTH_HEADER_NAME": "HTTP_AUTHORIZATION",
            "USER_ID_FIELD": "id",
            "USER_ID_CLAIM": "user_id",
            "USER_AUTHENTICATION_RULE": "rest_framework_simplejwt.authentication.default_user_authentication_rule",

            "AUTH_TOKEN_CLASSES": ("rest_framework_simplejwt.tokens.AccessToken",),
            "TOKEN_TYPE_CLAIM": "token_type",
            "TOKEN_USER_CLASS": "rest_framework_simplejwt.models.TokenUser",
            "JTI_CLAIM": "jti",

            'ROTATE_REFRESH_TOKENS': True,
            'ACCESS_TOKEN_LIFETIME': getattr(settings, 'JWT_ACCESS_TOKEN_LIFETIME', timedelta(minutes=30)),
            'REFRESH_TOKEN_LIFETIME': getattr(settings, 'JWT_REFRESH_TOKEN_LIFETIME', timedelta(days=90))
        }
        if not hasattr(settings, 'SIMPLE_JWT'):
            settings.SIMPLE_JWT = simple_jwt_settings
        else:
            for k in simple_jwt_settings.keys():
                if k not in settings.SIMPLE_JWT:
                    settings.SIMPLE_JWT[k] = simple_jwt_settings[k]

        if not hasattr(settings, 'REST_FRAMEWORK'):
            settings.REST_FRAMEWORK = {
                'DEFAULT_AUTHENTICATION_CLASSES': (
                    'rest_framework_simplejwt.authentication.JWTStatelessUserAuthentication',
                )
            }
        elif 'DEFAULT_AUTHENTICATION_CLASSES' not in settings.REST_FRAMEWORK:
            settings.REST_FRAMEWORK['DEFAULT_AUTHENTICATION_CLASSES'] = (
                'rest_framework_simplejwt.authentication.JWTStatelessUserAuthentication',
            )

        if not hasattr(settings, 'AUTHENTICATION_BACKENDS'):
            settings.AUTHENTICATION_BACKENDS = (
                # Needed to login by username in Django admin, regardless of `allauth`
                "django.contrib.auth.backends.ModelBackend",
                # `allauth` specific authentication methods, such as login by e-mail
                "allauth.account.auth_backends.AuthenticationBackend"
            )

        if "allauth.account.middleware.AccountMiddleware" not in settings.MIDDLEWARE:
            settings.MIDDLEWARE += ["allauth.account.middleware.AccountMiddleware"]

        reload(rest_framework_simplejwt.settings)
        reload(allauth.app_settings)



from django.conf import settings
from django.contrib.auth.forms import SetPasswordForm
from django.utils.translation import gettext_lazy as _
from rest_framework import serializers

from jwt_allauth.tokens.models import RefreshTokenWhitelistModel


class PasswordChangeSerializer(serializers.Serializer):
    old_password = serializers.CharField(max_length=128, write_only=True)
    new_password1 = serializers.CharField(max_length=128, write_only=True)
    new_password2 = serializers.CharField(max_length=128, write_only=True)

    set_password_form_class = SetPasswordForm

    def __init__(self, *args, **kwargs):
        self.old_password_field_enabled = getattr(
            settings, 'OLD_PASSWORD_FIELD_ENABLED', True
        )
        self.logout_on_password_change = getattr(
            settings, 'LOGOUT_ON_PASSWORD_CHANGE', True
        )
        super(PasswordChangeSerializer, self).__init__(*args, **kwargs)

        if not self.old_password_field_enabled:
            self.fields.pop('old_password')

        self.request = self.context.get('request')
        self.user = getattr(self.request, 'user', None)

    def validate_old_password(self, value):
        invalid_password_conditions = (
            self.old_password_field_enabled,
            self.user,
            not self.user.check_password(value)
        )

        if all(invalid_password_conditions):
            err_msg = _("Your old password was entered incorrectly. Please enter it again.")
            raise serializers.ValidationError(err_msg)
        return value

    def validate(self, attrs):
        self.set_password_form = self.set_password_form_class(
            user=self.user, data=attrs
        )

        if not self.set_password_form.is_valid():
            raise serializers.ValidationError(self.set_password_form.errors)
        return attrs

    def save(self):
        self.set_password_form.save()
        if self.logout_on_password_change:
            RefreshTokenWhitelistModel.objects.filter(user=self.request.user.id).exclude(
                session=self.request.auth['session']
            ).delete()
        else:
            from django.contrib.auth import update_session_auth_hash
            update_session_auth_hash(self.request, self.user)



import logging

from allauth.account import app_settings as allauth_settings
# from allauth.account.adapter import get_adapter
from allauth.account.utils import complete_signup
# from allauth.socialaccount import signals
# from allauth.socialaccount.adapter import get_adapter as get_social_adapter
# from allauth.socialaccount.models import SocialAccount
from django.conf import settings
from django.utils.translation import gettext_lazy as _
from rest_framework import status
# from rest_framework.exceptions import NotFound
from rest_framework.generics import CreateAPIView  #, ListAPIView, GenericAPIView
# from rest_framework.permissions import IsAuthenticated
from rest_framework.response import Response

# from jwt_allauth.login.views import LoginView
from jwt_allauth.tokens.models import TokenModel
from jwt_allauth.registration.app_settings import register_permission_classes
from jwt_allauth.app_settings import RegisterSerializer
from jwt_allauth.tokens.app_settings import RefreshToken
# from jwt_allauth.registration.serializers import (
#     SocialLoginSerializer, SocialAccountSerializer, SocialConnectSerializer)
from jwt_allauth.utils import get_user_agent, sensitive_post_parameters_m

logger = logging.getLogger(__name__)


class RegisterView(CreateAPIView):
    serializer_class = RegisterSerializer
    permission_classes = register_permission_classes()
    token_model = TokenModel
    jwt_token = RefreshToken

    @sensitive_post_parameters_m
    def dispatch(self, *args, **kwargs):
        return super(RegisterView, self).dispatch(*args, **kwargs)

    @staticmethod
    def get_response_data(token):
        if settings.EMAIL_VERIFICATION:
            return {
                "detail": _("Verification e-mail sent."),
                'refresh': str(token)
            }

        else:
            return {
                'refresh': str(token),
                'access': str(token.access_token)
            }

    @get_user_agent
    def create(self, request, *args, **kwargs):
        serializer = self.get_serializer(data=request.data)
        serializer.is_valid(raise_exception=True)
        token = self.perform_create(serializer)
        headers = self.get_success_headers(serializer.data)

        return Response(self.get_response_data(token),
                        status=status.HTTP_201_CREATED,
                        headers=headers)

    def perform_create(self, serializer):
        user = serializer.save(self.request)
        refresh = self.jwt_token.for_user(
            user, self.request, enabled=not bool(settings.EMAIL_VERIFICATION))

        # try:
        complete_signup(self.request._request, user,
                        allauth_settings.EMAIL_VERIFICATION,
                        None)
        return refresh


# class SocialLoginView(LoginView):
#     """
#     class used for social authentications
#     example usage for facebook with access_token
#     -------------
#     from allauth.socialaccount.providers.facebook.views import FacebookOAuth2Adapter
#
#     class FacebookLogin(SocialLoginView):
#         adapter_class = FacebookOAuth2Adapter
#     -------------
#
#     example usage for facebook with code
#
#     -------------
#     from allauth.socialaccount.providers.facebook.views import FacebookOAuth2Adapter
#     from allauth.socialaccount.providers.oauth2.client import OAuth2Client
#
#     class FacebookLogin(SocialLoginView):
#         adapter_class = FacebookOAuth2Adapter
#         client_class = OAuth2Client
#         callback_url = 'localhost:8000'
#     -------------
#     """
#     serializer_class = SocialLoginSerializer
#
#     def process_login(self):
#         get_adapter(self.request).login(self.request, self.user)
#
#
# class SocialConnectView(LoginView):
#     """
#     class used for social account linking
#
#     example usage for facebook with access_token
#     -------------
#     from allauth.socialaccount.providers.facebook.views import FacebookOAuth2Adapter
#
#     class FacebookConnect(SocialConnectView):
#         adapter_class = FacebookOAuth2Adapter
#     -------------
#     """
#     serializer_class = SocialConnectSerializer
#     permission_classes = (IsAuthenticated,)
#
#     def process_login(self):
#         get_adapter(self.request).login(self.request, self.user)
#
#
# class SocialAccountListView(ListAPIView):
#     """
#     List SocialAccounts for the currently logged in user
#     """
#     serializer_class = SocialAccountSerializer
#     permission_classes = (IsAuthenticated,)
#
#     def get_queryset(self):
#         return SocialAccount.objects.filter(user=self.request.user)
#
#
# class SocialAccountDisconnectView(GenericAPIView):
#     """
#     Disconnect SocialAccount from remote service for
#     the currently logged in user
#     """
#     serializer_class = SocialConnectSerializer
#     permission_classes = (IsAuthenticated,)
#
#     def get_queryset(self):
#         return SocialAccount.objects.filter(user=self.request.user)
#
#     def post(self, request, *args, **kwargs):
#         accounts = self.get_queryset()
#         account = accounts.filter(pk=kwargs['pk']).first()
#         if not account:
#             raise NotFound
#
#         get_social_adapter(self.request).validate_disconnect(account, accounts)
#
#         account.delete()
#         signals.social_account_removed.send(
#             sender=SocialAccount,
#             request=self.request,
#             socialaccount=account
#         )
#
#         return Response(self.get_serializer(account).data)



from importlib import import_module

from allauth.account.adapter import get_adapter
from allauth.account.models import EmailAddress
from django.contrib.auth import get_user_model
from django.utils.decorators import method_decorator
from django.views.decorators.debug import sensitive_post_parameters
from django.conf import settings

from django_user_agents.utils import get_user_agent as get_user_agent_django
from rest_framework_simplejwt.exceptions import InvalidToken
from six import string_types

from jwt_allauth.constants import TEMPLATE_PATHS
from jwt_allauth.exceptions import NotVerifiedEmail, IncorrectCredentials


def import_callable(path_or_callable):
    """
    Convert a Python path string to a callable object or return the input if already callable.

    Args:
        path_or_callable (str|callable): Either a Python path string (module.attribute)
                                        or an already callable object

    Returns:
        callable: The resolved callable object

    Raises:
        AssertionError: If input is string but not valid Python path
    """
    if hasattr(path_or_callable, '__call__'):
        return path_or_callable
    else:
        assert isinstance(path_or_callable, string_types)
        package, attr = path_or_callable.rsplit('.', 1)
        return getattr(import_module(package), attr)


def get_client_ip(request):
    """
    Extract client IP address from request metadata.

    Priority:

        1. X-Forwarded-For header (first entry if multiple)
        2. REMOTE_ADDR meta value

    Args:
        request (HttpRequest): Django request object

    Returns:
        str: Client IP address or None if not found
    """
    x_forwarded_for = request.META.get('HTTP_X_FORWARDED_FOR')
    if x_forwarded_for:
        ip = x_forwarded_for.split(',')[0]
    else:
        ip = request.META.get('REMOTE_ADDR')
    return ip


def get_user_agent(f):
    """
    Decorator that adds user agent and IP information to the request object.

    Stores:
    - user_agent: Parsed user agent details
    - ip: Client IP address

    Args:
        f (function): View method to decorate

    Returns:
        function: Decorated view method
    """
    def user_agent(self, request, *args, **kwargs):
        if getattr(settings, 'JWT_ALLAUTH_COLLECT_USER_AGENT', False):
            request.user_agent = get_user_agent_django(request)
            request.ip = get_client_ip(request)
        else:
            request.user_agent = None
            request.ip = None
        return f(self, request, *args, **kwargs)

    return user_agent


def user_agent_dict(request):
    """
    Generate a detailed dictionary of user agent information.

    Includes:

        - Browser details (name, version)
        - OS details (name, version)
        - Device information (family, brand, model)
        - Network information (IP address)
        - Device type flags (mobile, tablet, PC, bot)

    Args:
        request (HttpRequest): Django request object

    Returns:
        dict: Structured user agent details. Empty dict if no request.
    """
    if request is None:
        return {}
    if request.user_agent is None:
        return {}
    return {
        'browser': request.user_agent.browser.family,
        'browser_version': request.user_agent.browser.version_string,
        'os': request.user_agent.os.family,
        'os_version': request.user_agent.os.version_string,
        'device': request.user_agent.device.family,
        'device_brand': request.user_agent.device.brand,
        'device_model': request.user_agent.device.model,
        'ip': request.ip,
        'is_mobile': request.user_agent.is_mobile,
        'is_tablet': request.user_agent.is_tablet,
        'is_pc': request.user_agent.is_pc,
        'is_bot': request.user_agent.is_bot,
    }


sensitive_post_parameters_m = method_decorator(
    sensitive_post_parameters(
        'password', 'old_password', 'new_password1', 'new_password2', 'password1', 'password2'
    )
)


def get_template_path(constant, default):
    """
    Get template path from settings using TEMPLATE_PATHS configuration.

    Args:
        constant (str): Key to look up in TEMPLATE_PATHS setting
        default (str): Default path if not found in settings

    Returns:
        str: Configured template path or default value
    """
    templates_path_dict = getattr(settings, TEMPLATE_PATHS, {})
    return getattr(templates_path_dict, constant, default)


def is_email_verified(user, raise_exception=False):
    """
    Check if user has a verified email address.

    Args:
        user (User): User object to check
        raise_exception (bool): Whether to raise NotVerifiedEmail if unverified

    Returns:
        bool: True if verified, False otherwise

    Raises:
        NotVerifiedEmail: If raise_exception=True and email is unverified
    """
    if not EmailAddress.objects.filter(user=user.id, verified=True).exists():
        if raise_exception:
            raise NotVerifiedEmail()
        return False
    return True


def allauth_authenticate(**kwargs):
    """
    Authenticate user using allauth's adapter with enhanced verification.

    Args:
        **kwargs: Authentication credentials (typically username/email + password)

    Returns:
        User: Authenticated user object

    Raises:
        IncorrectCredentials: If authentication fails
        NotVerifiedEmail: If email is not verified
    """
    user = get_adapter().authenticate(**kwargs)
    if user is None:
        raise IncorrectCredentials()
    is_email_verified(user, raise_exception=True)
    return user


def load_user(f):
    """
    Decorator that loads the complete user object from the database for stateless JWT authentication.
    This is necessary because JWT tokens only contain the user ID, and the full user object
    might be needed in the view methods.

    Usage:

    .. code-block:: python

        @load_user
        def my_view_method(self, *args, **kwargs):
            # self.request.user will be the complete user object
            pass
    """
    def wrapper(self, *args, **kwargs):
        try:
            self.request.user = get_user_model().objects.get(id=self.request.user.id)
        except get_user_model().DoesNotExist:
            raise InvalidToken()
        res = f(self, *args, **kwargs)
        return res
    return wrapper



from django.utils.translation import gettext_lazy as _
from django.conf import settings
from rest_framework import status
from rest_framework.permissions import IsAuthenticated
from rest_framework.response import Response
from rest_framework.views import APIView
from rest_framework_simplejwt.exceptions import TokenError, InvalidToken

from jwt_allauth.logout.serializers import RemoveRefreshTokenSerializer
from jwt_allauth.tokens.models import RefreshTokenWhitelistModel
from jwt_allauth.constants import REFRESH_TOKEN_COOKIE


class LogoutView(APIView):
    """
    Calls Django logout method and delete the Token object
    assigned to the current User object.

    Accepts/Returns nothing.
    """
    permission_classes = (IsAuthenticated,)

    def get(self, request, *args, **kwargs):
        return self.http_method_not_allowed(request, *args, **kwargs)

    def post(self, request):
        return self.logout(request)

    @staticmethod
    def logout(request):
        data = request.data.copy()
        
        if getattr(settings, 'JWT_ALLAUTH_REFRESH_TOKEN_AS_COOKIE', False):
            refresh_token = request.COOKIES.get(REFRESH_TOKEN_COOKIE)
            if refresh_token:
                data['refresh'] = refresh_token
            else:
                return Response(
                    {"detail": _("Refresh token cookie not found.")},
                    status=status.HTTP_400_BAD_REQUEST
                )
        
        try:
            RemoveRefreshTokenSerializer(
                data=data,
                context={'user': request.user.id}
            ).is_valid(raise_exception=True)
            return Response(
                {"detail": _("Successfully logged out.")},
                status=status.HTTP_200_OK
            )
        except (TokenError, InvalidToken):
            return Response(
                {"detail": _("Invalid token.")},
                status=status.HTTP_400_BAD_REQUEST
            )


class LogoutAllView(APIView):
    """
    Logout from all devices.

    Accepts/Returns nothing.
    """
    permission_classes = (IsAuthenticated,)

    def get(self, request, *args, **kwargs):
        return self.http_method_not_allowed(request, *args, **kwargs)

    def post(self, request):
        return self.logout(request)

    @staticmethod
    def logout(request):
        RefreshTokenWhitelistModel.objects.filter(user=request.user.id).delete()
        return Response(
            {"detail": _("Successfully logged out from all devices.")},
            status=status.HTTP_200_OK
        )



from django.conf import settings
from rest_framework import status
from rest_framework.request import Request
from rest_framework.response import Response
from rest_framework_simplejwt.exceptions import TokenError, InvalidToken
from rest_framework_simplejwt.views import TokenRefreshView as DefaultTokenRefreshView
from rest_framework.throttling import UserRateThrottle
from jwt_allauth.token_refresh.serializers import TokenRefreshSerializer
from jwt_allauth.utils import get_user_agent, user_agent_dict
from jwt_allauth.constants import REFRESH_TOKEN_COOKIE


class TokenRefreshView(DefaultTokenRefreshView):
    serializer_class = TokenRefreshSerializer
    throttle_classes = [UserRateThrottle]

    @get_user_agent
    def post(self, request: Request, *args, **kwargs) -> Response:
        input_data = {}

        # Get refresh token from cookie or request data based on configuration
        if getattr(settings, 'JWT_ALLAUTH_REFRESH_TOKEN_AS_COOKIE', True):
            refresh_token = request.COOKIES.get('refresh_token')
            if refresh_token:
                input_data['refresh'] = refresh_token
        else:
            if 'refresh' in request.data:
                input_data['refresh'] = request.data['refresh']

        context = user_agent_dict(self.request)
        serializer = self.get_serializer(data=input_data, context=context)

        try:
            serializer.is_valid(raise_exception=True)
        except TokenError as e:
            raise InvalidToken(e.args[0])

        response_data = {"access": serializer.validated_data['access']}

        # Handle refresh token based on configuration
        if not getattr(settings, 'JWT_ALLAUTH_REFRESH_TOKEN_AS_COOKIE', True):
            response_data["refresh"] = serializer.validated_data['refresh']

        response = Response(response_data, status=status.HTTP_200_OK)

        if getattr(settings, 'JWT_ALLAUTH_REFRESH_TOKEN_AS_COOKIE', True):
            response.set_cookie(
                key=REFRESH_TOKEN_COOKIE,
                value=str(serializer.validated_data['refresh']),
                httponly=True,
                secure=not settings.DEBUG if hasattr(settings, 'DEBUG') else True,
                samesite='Lax'
            )

        return response



from typing import Union

from django.utils.functional import cached_property
from rest_framework_simplejwt.models import TokenUser

from jwt_allauth.constants import FOR_USER


class SetPasswordTokenUser(TokenUser):
    @cached_property
    def id(self) -> Union[int, str]:
        return self.token[FOR_USER]



from django.conf import settings
from django.contrib.auth import get_user_model
from django.core.exceptions import ImproperlyConfigured, ValidationError
from django.http import HttpResponseRedirect, HttpResponseNotFound
from django.shortcuts import render
from django.urls import reverse_lazy
from django.utils.http import urlsafe_base64_decode
from django.utils.translation import gettext_lazy as _
from rest_framework import status
from rest_framework.generics import GenericAPIView
from rest_framework.permissions import AllowAny
from rest_framework.response import Response
from rest_framework.throttling import AnonRateThrottle, UserRateThrottle
from rest_framework_simplejwt.exceptions import InvalidToken

from jwt_allauth.app_settings import PasswordResetSerializer
from jwt_allauth.constants import PASS_RESET, PASSWORD_RESET_REDIRECT, PASS_RESET_ACCESS, PASS_RESET_COOKIE, FOR_USER, \
    ONE_TIME_PERMISSION, REFRESH_TOKEN_COOKIE
from jwt_allauth.password_reset.permissions import ResetPasswordPermission
from jwt_allauth.password_reset.serializers import SetPasswordSerializer
from jwt_allauth.tokens.app_settings import RefreshToken
from jwt_allauth.tokens.models import GenericTokenModel, RefreshTokenWhitelistModel
from jwt_allauth.tokens.serializers import GenericTokenModelSerializer
from jwt_allauth.tokens.tokens import GenericToken
from jwt_allauth.utils import get_user_agent, sensitive_post_parameters_m


class PasswordResetView(GenericAPIView):
    """
    Calls Django Auth PasswordResetForm save method.

    Accepts the following POST parameters: email
    Returns the success/fail message.
    """
    serializer_class = PasswordResetSerializer
    permission_classes = (AllowAny,)
    throttle_classes = [AnonRateThrottle]

    @get_user_agent
    def post(self, request):
        # Create a serializer with request.data
        serializer = self.get_serializer(data=request.data)
        serializer.is_valid(raise_exception=True)
        serializer.save()
        # Return the success message with OK HTTP status
        return Response(
            {"detail": _("Password reset e-mail has been sent.")},
            status=status.HTTP_200_OK
        )


class DefaultPasswordResetView(GenericAPIView):
    """
    Default view for password reset form.
    """
    permission_classes = (AllowAny,)
    template_name = 'password/reset.html'

    def get(self, request):
        return render(request, self.template_name, {
            'validlink': PASS_RESET_COOKIE in request.COOKIES,
            'form': None
        })


class PasswordResetConfirmView(GenericAPIView):
    form_url = getattr(settings, PASSWORD_RESET_REDIRECT, None)

    @get_user_agent
    def get(self, *_, **kwargs):
        if "uidb64" not in kwargs or "token" not in kwargs:
            raise ImproperlyConfigured(
                "The URL path must contain 'uidb64' and 'token' parameters."
            )

        user = self.get_user(kwargs["uidb64"])

        if user is not None:
            if GenericToken(request=self.request, purpose=PASS_RESET).check_token(user, kwargs["token"]):

                refresh_token = RefreshToken()
                refresh_token[FOR_USER] = user.id
                refresh_token[ONE_TIME_PERMISSION] = PASS_RESET_ACCESS
                access_token = refresh_token.access_token

                response = HttpResponseRedirect(
                    self.form_url if self.form_url else reverse_lazy('default_password_reset')
                )
                response.set_cookie(
                    key=PASS_RESET_COOKIE,
                    value=str(access_token),
                    httponly=getattr(settings, 'PASSWORD_RESET_COOKIE_HTTP_ONLY', True),
                    secure=getattr(settings, 'PASSWORD_RESET_COOKIE_SECURE', not settings.DEBUG),
                    samesite=getattr(settings, 'PASSWORD_RESET_COOKIE_SAME_SITE', 'Lax'),
                    max_age=getattr(settings, 'PASSWORD_RESET_COOKIE_MAX_AGE', 3600)
                )

                token_serializer = GenericTokenModelSerializer(data={
                    'token': access_token['jti'],
                    'user': user.id,
                    'purpose': PASS_RESET_ACCESS
                })
                token_serializer.is_valid(raise_exception=True)
                token_serializer.save()

                return response
        return render(self.request, 'password/reset.html', {
            'validlink': False,
            'form': None
        })

    @staticmethod
    def get_user(uidb64):
        try:
            # urlsafe_base64_decode() decodes to bytestring
            uid = urlsafe_base64_decode(uidb64).decode()
            user = get_user_model()._default_manager.get(pk=uid)
        except (
            TypeError,
            ValueError,
            OverflowError,
            get_user_model().DoesNotExist,
            ValidationError,
        ):
            user = None
        return user


class ResetPasswordView(GenericAPIView):
    """
    Calls Django Auth SetPasswordForm save method.

    Accepts the following POST parameters: new_password1, new_password2
    Returns the success/fail message.
    """
    serializer_class = SetPasswordSerializer
    permission_classes = (ResetPasswordPermission,)
    throttle_classes = [UserRateThrottle]

    @sensitive_post_parameters_m
    def dispatch(self, *args, **kwargs):
        return super(ResetPasswordView, self).dispatch(*args, **kwargs)

    def post(self, request):
        # check the token has not been used
        query_set = GenericTokenModel.objects.filter(token=request.auth['jti'], purpose=PASS_RESET_ACCESS)
        if len(query_set) != 1:
            raise InvalidToken()
        query_set.delete()  # single use

        # Load the user in the request
        request.user = get_user_model().objects.get(id=self.request.user.id)
        serializer = self.get_serializer(data=request.data)
        serializer.is_valid(raise_exception=True)
        serializer.save()

        # Revoke old sessions
        if getattr(settings, 'LOGOUT_ON_PASSWORD_CHANGE', True):
            RefreshTokenWhitelistModel.objects.filter(user=self.request.user.id).delete()

        refresh_token = RefreshToken.for_user(request.user)
        response_data = {
            "access": str(refresh_token.access_token),
            "detail": _("Password reset.")
        }

        # Handle refresh token based on configuration
        if not getattr(settings, 'JWT_ALLAUTH_REFRESH_TOKEN_AS_COOKIE', True):
            response_data["refresh"] = str(refresh_token)

        response = Response(response_data)

        if getattr(settings, 'JWT_ALLAUTH_REFRESH_TOKEN_AS_COOKIE', True):
            response.set_cookie(
                key=REFRESH_TOKEN_COOKIE,
                value=str(refresh_token),
                httponly=True,
                secure=not settings.DEBUG if hasattr(settings, 'DEBUG') else True,
                samesite='Lax'
            )

        return response



#!/usr/bin/env python3
"""
JWT Allauth CLI tool for creating and managing projects
"""
import sys
import os
import argparse
import subprocess
import re


def main():
    """
    Main entry point for the JWT Allauth CLI
    """
    parser = argparse.ArgumentParser(
        description='JWT Allauth command line tool',
        usage='jwt-allauth <command> [options]'
    )
    parser.add_argument('command', help='Command to run (e.g. startproject)')
    
    # Parse just the command argument first
    args, remaining_args = parser.parse_known_args()
    
    if args.command == 'startproject':
        # Handle startproject command
        project_parser = argparse.ArgumentParser(
            description='Create a new Django project with JWT Allauth pre-configured',
            usage='jwt-allauth startproject <name> [directory] [options]'
        )
        project_parser.add_argument('name', help='Name of the project')
        project_parser.add_argument('directory', nargs='?', help='Optional directory to create the project in')
        project_parser.add_argument('--email', default='False', help='Email configuration (True/False)')
        project_parser.add_argument('--template', help='Template directory to use as a base')
        
        project_args = project_parser.parse_args(remaining_args)
        
        # Get project arguments
        project_name = project_args.name
        target_dir = project_args.directory or project_name
        email_config = project_args.email.lower() == 'true'
        template = project_args.template
        
        try:
            # Build command for running django-admin startproject
            cmd = ["django-admin", "startproject", project_name]
            
            # Add directory if specified
            if project_args.directory:
                cmd.append(project_args.directory)
                
            # Add template if specified
            if template:
                cmd.extend(["--template", template])
            
            # Run django-admin startproject
            print(f"Creating Django project '{project_name}'...")
            result = subprocess.run(cmd, capture_output=True, text=True)
            
            if result.returncode != 0:
                print(f"❌ Error creating project: {result.stderr}")
                return 1
                
            print(f"✅ Created Django project '{project_name}'")
            
            # Step 2: Now modify the created project for JWT Allauth
            # Path to settings file
            settings_path = os.path.join(target_dir, project_name, 'settings.py')
            
            # Modify settings.py to include JWT-allauth configuration
            _modify_settings(settings_path, email_config)
            print("✅ Added JWT Allauth configuration to settings.py")
            
            # Add urls.py configuration
            urls_path = os.path.join(target_dir, project_name, 'urls.py')
            _modify_urls(urls_path)
            print("✅ Added JWT Allauth URLs to urls.py")
            
            # Create templates directory if needed
            if email_config:
                templates_dir = os.path.join(target_dir, 'templates')
                os.makedirs(templates_dir, exist_ok=True)
                print("✅ Created templates directory")
            
            # Final instructions
            print("\n✅ JWT Allauth project successfully created!")
            print("📋 Next steps:")
            print(f"   1. cd {target_dir}")
            print(f"   2. python manage.py makemigrations")
            print(f"   3. python manage.py migrate")
            print(f"   4. python manage.py runserver")
            
            if email_config:
                print("\n⚠️ Email configuration is enabled. Please update your email settings in settings.py")
            
        except Exception as e:
            import traceback
            traceback.print_exc()
            print(f"❌ Unexpected error: {str(e)}")
            return 1
    
    elif args.command == 'help':
        parser.print_help()
        print("\nAvailable commands:")
        print("  startproject  - Create a new Django project with JWT Allauth pre-configured")
        
    else:
        print(f"❓ Unknown command: {args.command}")
        parser.print_help()
        return 1
    
    return 0

def _modify_settings(settings_path, email_config):
    """Modify Django settings.py to include JWT Allauth configuration"""
    with open(settings_path, 'r') as f:
        settings_content = f.read()

    # Find INSTALLED_APPS content
    pattern = r"(INSTALLED_APPS\s*=\s*\[)(.*?)(,*\n*])"
    jwt_apps = """    'jwt_allauth',
    'rest_framework',
    'rest_framework.authtoken',
    'allauth',
    'allauth.account',
    'allauth.socialaccount',"""
    
    # Replace keeping original Django apps and adding new ones before closing bracket
    settings_content = re.sub(pattern, fr'\1\2,\n{jwt_apps}\n]', settings_content, flags=re.DOTALL)
    
    # Add middleware
    pattern = r"(MIDDLEWARE\s*=\s*\[)(.*?)(,*\n*\])"
    allauth_middleware = "    'allauth.account.middleware.AccountMiddleware',"
    # Replace keeping original middleware and adding new one before closing bracket
    settings_content = re.sub(pattern, fr'\1\2,\n{allauth_middleware}\n]', settings_content, flags=re.DOTALL)
    
    # Add authentication backends
    auth_backends = """
# JWT Allauth user model
AUTH_USER_MODEL = 'jwt_allauth.JAUser'

# JWT Allauth adapter
ACCOUNT_ADAPTER = 'jwt_allauth.adapter.JWTAllAuthAdapter'

# Login configuration
ACCOUNT_LOGIN_METHODS = {'email'}
ACCOUNT_SIGNUP_FIELDS = ['email*', 'password1*', 'password2*']

# Authentication backends
AUTHENTICATION_BACKENDS = (
    # Needed to login by username in Django admin, regardless of `allauth`
    "django.contrib.auth.backends.ModelBackend",
    # `allauth` specific authentication methods, such as login by e-mail
    "allauth.account.auth_backends.AuthenticationBackend"
)

# Django Rest Framework settings
REST_FRAMEWORK = {
    'DEFAULT_AUTHENTICATION_CLASSES': (
        'rest_framework_simplejwt.authentication.JWTStatelessUserAuthentication',
    )
}


from datetime import timedelta

# JWT settings
JWT_ACCESS_TOKEN_LIFETIME = timedelta(minutes=30)
JWT_REFRESH_TOKEN_LIFETIME = timedelta(days=90)
"""
    settings_content += auth_backends
    
    # Add email configuration if requested
    if email_config:
        email_settings = """
# Email configuration
EMAIL_VERIFICATION = True
EMAIL_BACKEND = 'django.core.mail.backends.smtp.EmailBackend'
EMAIL_HOST = 'smtp.example.com'  # Update with your SMTP server
EMAIL_PORT = 587
EMAIL_USE_TLS = True
EMAIL_HOST_USER = 'your-email@example.com'  # Update with your email
EMAIL_HOST_PASSWORD = 'your-password'  # Update with your password
DEFAULT_FROM_EMAIL = 'your-email@example.com'  # Update with your email

# JWT Allauth settings
EMAIL_VERIFIED_REDIRECT = None  # URL to redirect after email verification
PASSWORD_RESET_REDIRECT = None  # URL for password reset form
"""
        settings_content += email_settings
    
    with open(settings_path, 'w') as f:
        f.write(settings_content)

def _modify_urls(urls_path):
    """Modify Django urls.py to include JWT Allauth URLs"""
    with open(urls_path, 'r') as f:
        urls_content = f.read()

    no_comments_urls_content = re.sub(r'""".*?"""', '', urls_content, flags=re.DOTALL)

    # Add import for include if needed
    if "from django.urls import path" in no_comments_urls_content:
        if "include" not in no_comments_urls_content:
            urls_content = urls_content.replace(
                "from django.urls import path",
                "from django.urls import path, include"
            )
    elif "include" not in no_comments_urls_content:
        urls_content = "from django.urls import include\n" + urls_content
    
    # Add JWT-allauth URLs
    urls_pattern = r"(urlpatterns\s*=\s*\[)(.*?)(,*\n*\])"
    jwt_urls = "    path('jwt-allauth/', include('jwt_allauth.urls')),"
    urls_content = re.sub(urls_pattern, fr'\1\2,\n{jwt_urls}\n]', urls_content, flags=re.DOTALL)

    with open(urls_path, 'w') as f:
        f.write(urls_content)

if __name__ == '__main__':
    sys.exit(main())



from allauth.account.internal.userkit import user_field
from allauth.account.models import EmailAddress
from django.contrib.auth import get_user_model
from django.test import TestCase
from django.test.client import Client

from jwt_allauth.tokens.app_settings import RefreshToken


class JAClient(Client):
    """
    A custom Django test client for handling JWT authenticated requests.

    Provides enhanced HTTP methods to automatically include JWT tokens in requests.
    Supports both regular user and staff user authentication through separate tokens.

    All standard HTTP methods (post, get, patch, put, delete) are extended with:

        - Regular auth versions (auth_* methods) using default user token
        - Staff auth versions (staff_* methods) using staff user token
        - Optional direct token injection via access_token parameter

    """
    content_type = 'application/json'

    def __init__(self, token, staff_token, *args, **kwargs):
        self.ACCESS = token
        self.STAFF_ACCESS = staff_token
        super().__init__(*args, **kwargs)

    def update_kwargs(self, access_token=None, default_auth=False, staff_auth=False, **kwargs):
        kwargs['content_type'] = self.content_type
        if access_token is not None:
            kwargs['HTTP_AUTHORIZATION'] = f'Bearer {access_token}'
        elif default_auth:
            kwargs['HTTP_AUTHORIZATION'] = f'Bearer {self.ACCESS}'
        elif staff_auth:
            kwargs['HTTP_AUTHORIZATION'] = f'Bearer {self.STAFF_ACCESS}'
        return kwargs

    def post(self, *args, access_token=None, **kwargs):
        kwargs = self.update_kwargs(access_token=access_token, **kwargs)
        return super().post(*args, **kwargs)

    def auth_post(self, *args, **kwargs):
        kwargs = self.update_kwargs(**kwargs, default_auth=True)
        return super().post(*args, **kwargs)

    def get(self, *args, access_token=None, **kwargs):
        kwargs = self.update_kwargs(access_token=access_token, **kwargs)
        return super().get(*args, **kwargs)

    def auth_get(self, *args, **kwargs):
        kwargs = self.update_kwargs(**kwargs, default_auth=True)
        return super().get(*args, **kwargs)

    def patch(self, *args, access_token=None, **kwargs):
        kwargs = self.update_kwargs(access_token=access_token, **kwargs)
        return super().patch(*args, **kwargs)

    def auth_patch(self, *args, **kwargs):
        kwargs = self.update_kwargs(**kwargs, default_auth=True)
        return super().patch(*args, **kwargs)

    def put(self, *args, access_token=None, **kwargs):
        kwargs = self.update_kwargs(access_token=access_token, **kwargs)
        return super().put(*args, **kwargs)

    def auth_put(self, *args, **kwargs):
        kwargs = self.update_kwargs(**kwargs, default_auth=True)
        return super().put(*args, **kwargs)

    def delete(self, *args, access_token=None, **kwargs):
        kwargs = self.update_kwargs(access_token=access_token, **kwargs)
        return super().delete(*args, **kwargs)

    def auth_delete(self, *args, **kwargs):
        kwargs = self.update_kwargs(**kwargs, default_auth=True)
        return super().delete(*args, **kwargs)

    def staff_post(self, *args, **kwargs):
        kwargs = self.update_kwargs(**kwargs, staff_auth=True)
        return super().post(*args, **kwargs)

    def staff_get(self, *args, **kwargs):
        kwargs = self.update_kwargs(**kwargs, staff_auth=True)
        return super().get(*args, **kwargs)

    def staff_patch(self, *args, **kwargs):
        kwargs = self.update_kwargs(**kwargs, staff_auth=True)
        return super().patch(*args, **kwargs)

    def staff_put(self, *args, **kwargs):
        kwargs = self.update_kwargs(**kwargs, staff_auth=True)
        return super().put(*args, **kwargs)

    def staff_delete(self, *args, **kwargs):
        kwargs = self.update_kwargs(**kwargs, staff_auth=True)
        return super().delete(*args, **kwargs)


class JATestCase(TestCase):
    """
    Base test case for JWT-authenticated endpoint testing.

    Provides pre-configured user accounts and JWT tokens for testing:

        - Regular user with verified email
        - Staff user with verified email
        - Ready-to-use test client with authentication support
    """
    EMAIL = 'test@mail.com'
    PASS = 'Test-Passw0rd'
    FIRST_NAME = 'name'
    LAST_NAME = 'surname'
    USER = None
    LOGIN_PAYLOAD = {'email': EMAIL, 'password': PASS}
    STAFF_EMAIL = 'test@staff.com'
    STAFF_PASS = 'Staff-Passw0rd'
    STAFF_FIRST_NAME = 'staffname'
    STAFF_LAST_NAME = 'staffsurname'
    STAFF_USER = None

    def setUp(self):
        """
        Configures test environment with regular and staff users, including:

            - User account creation
            - Email verification setup
            - JWT token generation
        """
        self.USER = get_user_model().objects.create_user(self.EMAIL, email=self.EMAIL, password=self.PASS)
        user_field(self.USER, "first_name", self.FIRST_NAME)
        user_field(self.USER, "last_name", self.LAST_NAME)
        EmailAddress.objects.create(user=self.USER, email=self.EMAIL, verified=True, primary=True)
        self.TOKEN = RefreshToken().for_user(self.USER)
        self.ACCESS = str(self.TOKEN.access_token)
        self.USER.save()
        self.STAFF_USER = get_user_model().objects.create_user(
            self.STAFF_EMAIL, email=self.STAFF_EMAIL, password=self.STAFF_PASS, is_staff=True)
        user_field(self.STAFF_USER, "first_name", self.STAFF_FIRST_NAME)
        user_field(self.STAFF_USER, "last_name", self.STAFF_LAST_NAME)
        EmailAddress.objects.create(user=self.STAFF_USER, email=self.STAFF_EMAIL, verified=True, primary=True)
        self.STAFF_TOKEN = RefreshToken().for_user(self.STAFF_USER)
        self.STAFF_ACCESS = str(self.STAFF_TOKEN.access_token)
        self.STAFF_USER.save()

    @property
    def ja_client(self):
        """
        Pre-configured test client with authentication tokens.
        """
        return JAClient(self.ACCESS, self.STAFF_ACCESS)

    def authenticate(self, user):
        self.ACCESS = str(RefreshToken.for_user(user).access_token)



from allauth.account import app_settings as allauth_app_settings
from allauth.account.adapter import DefaultAccountAdapter
from allauth.core import context as allauth_ctx
from django.contrib.sites.shortcuts import get_current_site
from django.core.mail import EmailMessage, EmailMultiAlternatives
from django.template import TemplateDoesNotExist
from django.template.loader import render_to_string

from jwt_allauth.utils import get_template_path


class JWTAllAuthAdapter(DefaultAccountAdapter):
    """
    Custom account adapter extending allauth's DefaultAccountAdapter with JWT-specific email handling.

    Provides enhanced email confirmation functionality with template path customization
    and JWT-related email content handling.

    Key Features:

        - Email normalization (trimming and lowercasing)
        - Customizable template paths for verification emails
        - Dual template support (HTML/text) with fallback handling
        - Integration with JWT verification workflows
    """
    def clean_email(self, email):
        """
        Normalize email addresses by trimming whitespace and converting to lowercase.

        Args:
            email (str): Raw email input

        Returns:
            str: Normalized email address
        """
        email = super().clean_email(email)
        email = email.strip().lower()
        return email

    def send_confirmation_mail(self, request, emailconfirmation, signup):
        """
        Generate and send email confirmation message with context customization.

        Context Includes:

            - User object
            - Verification code or URL (based on EMAIL_VERIFICATION_BY_CODE_ENABLED)
            - Site-specific information

        Args:
            request (HttpRequest): Current request object
            emailconfirmation (EmailConfirmation): Email confirmation instance
            signup (bool): Flag indicating if this is a signup confirmation

        Returns:
            str: Confirmation key used in the email
        """
        ctx = {
            "user": emailconfirmation.email_address.user,
        }
        if allauth_app_settings.EMAIL_VERIFICATION_BY_CODE_ENABLED:
            ctx.update({"code": emailconfirmation.key})
        else:
            ctx.update(
                {
                    "key": emailconfirmation.key,
                    "activate_url": self.get_email_confirmation_url(
                        request, emailconfirmation
                    ),
                }
            )
        if signup:
            email_template = "email/signup/email_signup"
            template_path = get_template_path(
                'EMAIL_VERIFICATION',
                "email/signup/email_message.html"
            )
            subject_path = get_template_path(
                'EMAIL_VERIFICATION_SUBJECT',
                "email/signup/email_subject.txt"
            )
        else:
            email_template = "account/email/email_confirmation"
            template_path = None
            subject_path = None
        self.send_mail(
            email_template,
            emailconfirmation.email_address.email,
            ctx,
            subject_path=subject_path,
            template_path=template_path
        )
        return ctx['key']

    def send_mail(self, template_prefix, email, context, subject_path=None, template_path=None):
        """
        Construct and send email using template configuration.

        Enhances Context With:

            - Current site information
            - Recipient email address

        Args:
            template_prefix (str): Base path for template lookup
            email (str|list): Recipient email address(es)
            context (dict): Template context variables
            subject_path (str, optional): Custom path for subject template
            template_path (str, optional): Custom path for body template
        """
        ctx = {
            "email": email,
            "current_site": get_current_site(allauth_ctx.request),
        }
        ctx.update(context)
        msg = self.render_mail(template_prefix, email, ctx, subject_path=subject_path, template_path=template_path)
        msg.send()

    def render_mail(self, template_prefix, email, context, headers=None, subject_path=None, template_path=None):
        """
        Render email message with support for multiple template formats and custom paths.

        Behavior:

            - Generates multipart emails when both HTML and text templates exist
            - Uses custom template paths when provided
            - Automatically formats email subject
            - Supports HTML email content as primary when specified

        Args:
            template_prefix (str): Base template path prefix
            email (str|list): Recipient email address(es)
            context (dict): Template context variables
            headers (dict, optional): Custom email headers
            subject_path (str, optional): Override path for subject template
            template_path (str, optional): Override path for body template

        Returns:
            EmailMessage: Configured email message object

        Raises:
            TemplateDoesNotExist: If no valid template can be found
        """
        to = [email] if isinstance(email, str) else email
        if subject_path is None:
            subject_path = "{0}_subject.txt".format(template_prefix)
        subject = render_to_string(subject_path, context)
        # remove superfluous line breaks
        subject = " ".join(subject.splitlines()).strip()
        subject = self.format_email_subject(subject)

        from_email = self.get_from_email()

        if template_path is None:
            bodies = {}
            html_ext = allauth_app_settings.TEMPLATE_EXTENSION
            for ext in [html_ext, "txt"]:
                try:
                    template_name = "{0}_message.{1}".format(template_prefix, ext)
                    bodies[ext] = render_to_string(
                        template_name,
                        context,
                        allauth_ctx.request,
                    ).strip()
                except TemplateDoesNotExist:
                    if ext == "txt" and not bodies:
                        # We need at least one body
                        raise
        else:
            html_ext = 'html'
            bodies = {
                html_ext: render_to_string(
                    template_path,
                    context,
                    allauth_ctx.request,
                ).strip()
            }
        if "txt" in bodies:
            msg = EmailMultiAlternatives(
                subject, bodies["txt"], from_email, to, headers=headers
            )
            if html_ext in bodies:
                msg.attach_alternative(bodies[html_ext], "text/html")
        else:
            msg = EmailMessage(
                subject, bodies[html_ext], from_email, to, headers=headers
            )
            msg.content_subtype = "html"  # Main content is now text/html
        return msg



from typing import Dict, Any

from rest_framework import serializers
from rest_framework_simplejwt.exceptions import InvalidToken

from jwt_allauth.tokens.app_settings import RefreshToken
from jwt_allauth.tokens.models import RefreshTokenWhitelistModel
from jwt_allauth.tokens.serializers import RefreshTokenWhitelistSerializer
from jwt_allauth.utils import is_email_verified


class TokenRefreshSerializer(serializers.Serializer):
    refresh = serializers.CharField()
    access = serializers.CharField(read_only=True)
    token_class = RefreshToken

    def validate(self, attrs: Dict[str, Any]) -> Dict[str, str]:
        refresh = self.token_class(attrs["refresh"])

        query_set = list(RefreshTokenWhitelistModel.objects.filter(jti=refresh.payload["jti"]).all())
        if len(query_set) == 0:
            # Suspicious operation
            RefreshTokenWhitelistModel.objects.filter(session=refresh.payload["session"]).delete()
            raise InvalidToken()
        if not query_set[0].enabled:
            is_email_verified(query_set[0].user, raise_exception=True)
            raise InvalidToken()

        data = {"access": str(refresh.access_token)}

        RefreshTokenWhitelistModel.objects.filter(jti=refresh.payload["jti"]).delete()

        refresh.set_jti()
        refresh.set_exp()
        refresh.set_iat()

        data["refresh"] = str(refresh)

        del attrs["refresh"]

        # Get user agent data from context instead of request data
        serializer_data = {
            'jti': refresh.payload['jti'],
            'user': refresh.payload['user_id'],
            'session': refresh.payload['session'],
            **self.context  # Use context data here
        }

        refresh_serializer = RefreshTokenWhitelistSerializer(data=serializer_data)
        if refresh_serializer.is_valid():
            refresh_serializer.save()

        return data



from typing import Dict, Any

from django.conf import settings
from django.contrib.auth.models import update_last_login
from django.db import transaction
from rest_framework import exceptions
from rest_framework_simplejwt.serializers import TokenObtainPairSerializer
from rest_framework_simplejwt.settings import api_settings

from jwt_allauth.tokens.app_settings import RefreshToken
from jwt_allauth.utils import allauth_authenticate


class LoginSerializer(TokenObtainPairSerializer):
    token_class = RefreshToken
    username_field = getattr(settings, 'ACCOUNT_AUTHENTICATION_METHOD', 'email')
    user = None

    @classmethod
    def get_token(cls, user) -> RefreshToken:
        """
        Instantiates a new TokenObtainPairSerializer object, sets a token for the given user and returns the token.
        """
        cls.token = cls.token_class.for_user(user)
        return cls.token  # type: ignore

    @transaction.atomic
    def validate(self, attrs: Dict[str, Any]) -> Dict[Any, Any]:
        # Get the email and password information
        authenticate_kwargs = {
            self.username_field: attrs[self.username_field],
            "password": attrs["password"],
        }
        try:
            authenticate_kwargs["request"] = self.context["request"]
        except KeyError:
            pass

        # User authentication (allauth)
        self.user = allauth_authenticate(**authenticate_kwargs)

        # Active account check
        if not api_settings.USER_AUTHENTICATION_RULE(self.user):
            raise exceptions.AuthenticationFailed(
                self.error_messages["no_active_account"],
                "no_active_account",
            )
        validated_data = super().validate(attrs)

        # Set the refresh token
        refresh = self.get_token(self.user)

        validated_data["refresh"] = str(refresh)
        validated_data["access"] = str(refresh.access_token)

        if api_settings.UPDATE_LAST_LOGIN:
            update_last_login(None, self.user)

        return validated_data



from django.conf import settings
from django.db import models
from django.utils.translation import gettext_lazy as _
from rest_framework.authtoken.models import Token as DefaultTokenModel

from jwt_allauth.utils import import_callable

TokenModel = import_callable(
    getattr(settings, 'REST_AUTH_TOKEN_MODEL', DefaultTokenModel))


class BaseToken(models.Model):
    id = models.BigAutoField(primary_key=True)
    created = models.DateTimeField(_("created"), auto_now_add=True)
    ip = models.GenericIPAddressField(_("ip"), blank=True, null=True, max_length=39)
    is_mobile = models.BooleanField(_("is mobile"), null=True)
    is_tablet = models.BooleanField(_("is tablet"), null=True)
    is_pc = models.BooleanField(_("is pc"), null=True)
    is_bot = models.BooleanField(_("is bot"), null=True)
    browser = models.CharField(_("browser"), max_length=32, blank=True, null=True)
    browser_version = models.CharField(_("browser version"), max_length=32, blank=True, null=True)
    os = models.CharField(_("os"), max_length=32, blank=True, null=True)
    os_version = models.CharField(_("os version"), max_length=32, blank=True, null=True)
    device = models.CharField(_("device"), max_length=32, blank=True, null=True)
    device_brand = models.CharField(_("device brand"), max_length=32, blank=True, null=True)
    device_model = models.CharField(_("device model"), max_length=32, blank=True, null=True)

    class Meta:
        abstract = True
        verbose_name = _("refresh token")
        verbose_name_plural = _("refresh tokens")


class AbstractRefreshToken(BaseToken):
    jti = models.CharField(_("jti"), max_length=32, blank=False)
    enabled = models.BooleanField(_("enabled"), default=True)
    session = models.CharField(_("session"), max_length=32, blank=False)

    class Meta:
        abstract = True
        verbose_name = _("refresh token")
        verbose_name_plural = _("refresh tokens")


class RefreshTokenWhitelistModel(AbstractRefreshToken):
    user = models.ForeignKey(
        settings.AUTH_USER_MODEL,
        on_delete=models.CASCADE,
        related_name="refresh_tokens_whitelist",
        verbose_name=_("user"),
    )


class GenericTokenModel(BaseToken):
    user = models.ForeignKey(
        settings.AUTH_USER_MODEL,
        on_delete=models.CASCADE,
        related_name="generic_tokens",
        verbose_name=_("user"),
    )
    token = models.CharField(_("token"), max_length=255, blank=False)
    purpose = models.CharField(_("purpose"), max_length=32, blank=False)



from django.conf import settings

from rest_framework.permissions import AllowAny
from jwt_allauth.utils import import_callable


def register_permission_classes():
    permission_classes = [AllowAny, ]
    for klass in getattr(settings, 'JWT_ALLAUTH_REGISTER_PERMISSION_CLASSES', tuple()):
        permission_classes.append(import_callable(klass))
    return tuple(permission_classes)



from django.urls import path, include
from django.conf import settings
from django.views.generic import TemplateView

from jwt_allauth.login.views import LoginView
from jwt_allauth.logout.views import LogoutView, LogoutAllView
from jwt_allauth.password_change.views import PasswordChangeView
from jwt_allauth.password_reset.views import PasswordResetView, PasswordResetConfirmView, ResetPasswordView, DefaultPasswordResetView
from jwt_allauth.registration import urls as registration_urls
from jwt_allauth.token_refresh.views import TokenRefreshView
from jwt_allauth.user_details.views import UserDetailsView


urlpatterns = [
    # URLs that do not require a session or valid token
    path('password/reset/', PasswordResetView.as_view(), name='rest_password_reset'),
    path('password/reset/set-new/', ResetPasswordView.as_view(), name='rest_password_reset_set_new'),
    path(
        'password/reset/confirm/<str:uidb64>/<str:token>/',
        PasswordResetConfirmView.as_view(),
        name='password_reset_confirm'
    ),
    path('login/', LoginView.as_view(), name='rest_login'),
    path('refresh/', TokenRefreshView.as_view(), name='token_refresh'),
    # URLs that require a user to be logged in with a valid session / token.
    path('logout/', LogoutView.as_view(), name='rest_logout'),
    path('logout-all/', LogoutAllView.as_view(), name='rest_logout_all'),
    path('user/', UserDetailsView.as_view(), name='rest_user_details'),
    path('password/change/', PasswordChangeView.as_view(), name='rest_password_change'),
    # Registration urls
    path('registration/', include(registration_urls)),
]

if getattr(settings, 'PASSWORD_RESET_REDIRECT', None) is None:
    urlpatterns.append(
        path('password/reset/default/', DefaultPasswordResetView.as_view(), name='default_password_reset')
    )
    urlpatterns.append(
        path('password/reset/complete/', TemplateView.as_view(
            template_name='password/reset_complete.html'
        ), name='jwt_allauth_password_reset_complete'),
    )



import hashlib
from django.conf import settings
from uuid import uuid4
from inspect import getattr_static

from django.contrib.auth.tokens import PasswordResetTokenGenerator
from rest_framework.exceptions import ValidationError
from rest_framework_simplejwt.exceptions import InvalidToken
from rest_framework_simplejwt.tokens import RefreshToken as DefaultRefreshToken

from jwt_allauth.tokens.models import GenericTokenModel
from jwt_allauth.tokens.serializers import RefreshTokenWhitelistSerializer, GenericTokenModelSerializer
from jwt_allauth.utils import user_agent_dict


class RefreshToken(DefaultRefreshToken):

    def set_session(self, id_=None):
        """
        Unique identifier of the session associated to the refresh token.
        """
        if id_ is None:
            id_ = uuid4().hex
        self.payload['session'] = id_

    def set_user_attributes(self, user):
        """
        Add configurable user attributes to the token payload.
        Expects settings.JWT_ALLAUTH_USER_ATTRIBUTES as a dict mapping
        output claim names to dot-paths on the user object.
        Example: {'organization_id': 'organization.id', 'area_id': 'area.id'}
        """
        configured_attributes = getattr(settings, 'JWT_ALLAUTH_USER_ATTRIBUTES', {})

        # Accept legacy list format for backward compatibility but prefer dict.
        # In legacy mode, the final attribute name is used as the claim key.
        if isinstance(configured_attributes, list):
            attribute_map = {
                attr_path.split('.')[-1]: attr_path
                for attr_path in configured_attributes
            }
        elif isinstance(configured_attributes, dict):
            attribute_map = configured_attributes
        else:
            attribute_map = {}

        # Validate configuration: output names must be unique and must not collide
        # with reserved payload keys like 'role'.
        output_names = list(attribute_map.keys())
        duplicates = sorted(set([name for name in output_names if output_names.count(name) > 1]))
        reserved_conflicts = []
        if 'role' in output_names:
            reserved_conflicts.append('role')
        if duplicates or reserved_conflicts:
            conflict_list = sorted(set(duplicates + reserved_conflicts))
            raise ValueError(
                f"Incompatible JWT_ALLAUTH_USER_ATTRIBUTES: duplicate or reserved attribute names {conflict_list}"
            )

        for output_name, attr_path in attribute_map.items():
            keys = attr_path.split('.')
            current_value = user
            missing = False

            for key in keys:
                # First, verify the attribute exists without triggering __getattr__
                try:
                    getattr_static(current_value, key)
                except AttributeError:
                    missing = True
                    break

                # Then, resolve the runtime value (to handle descriptors/properties)
                try:
                    current_value = getattr(current_value, key)
                except Exception:
                    missing = True
                    break

                if current_value is None:
                    missing = True
                    break

            if not missing and output_name != 'role' and output_name not in self.payload and not callable(current_value):
                self.payload[output_name] = current_value

    def set_user_role(self, user):
        self.payload['role'] = user.role

    @classmethod
    def for_user(cls, user, request=None, enabled=True):
        """
        Return
        ------
        RefreshToken

        """
        token = super().for_user(user)
        token.set_session()  # type: ignore
        token.set_user_role(user)  # type: ignore
        token.set_user_attributes(user)  # type: ignore
        # Store the token in the database
        refresh_serializer = RefreshTokenWhitelistSerializer(data={
            'jti': token.payload['jti'],
            'user': user.id,
            'enabled': enabled,
            'session': token.payload['session'],
            **user_agent_dict(request)
        })
        try:
            refresh_serializer.is_valid(raise_exception=True)
            refresh_serializer.save()
        except ValidationError as e:
            raise InvalidToken(e.args[0])
        return token


class GenericToken(PasswordResetTokenGenerator):

    def __init__(self, purpose, request=None):
        super().__init__()
        self.request = request
        self.purpose = purpose

    def make_token(self, user):
        token = super().make_token(user)
        hashed_token = hashlib.sha256(str(token).encode()).hexdigest()
        token_serializer = GenericTokenModelSerializer(data={
            'token': hashed_token,
            'user': user.id,
            'purpose': self.purpose,
            **user_agent_dict(self.request)
        })
        try:
            token_serializer.is_valid(raise_exception=True)
            token_serializer.save()
            # remove existing tokens for the same purpose
            GenericTokenModel.objects.filter(user=user, purpose=self.purpose).exclude(token=hashed_token).delete()
        except ValidationError as e:
            raise InvalidToken(e.args[0])
        return token

    def check_token(self, user, token):
        result = super().check_token(user, token)
        if result:
            hashed_token = hashlib.sha256(str(token).encode()).hexdigest()
            if GenericTokenModel.objects.filter(token=hashed_token, purpose=self.purpose).count() == 0:
                return False
            GenericTokenModel.objects.filter(token=hashed_token, purpose=self.purpose).delete()
        return result



PASS_RESET = 'PASS_RESET'
PASS_RESET_ACCESS = 'PASS_RESET_ACCESS'
TEMPLATE_PATHS = 'JWT_ALLAUTH_TEMPLATES'

EMAIL_VERIFIED_REDIRECT = 'EMAIL_VERIFIED_REDIRECT'
PASSWORD_RESET_REDIRECT = 'PASSWORD_RESET_REDIRECT'

PASS_RESET_COOKIE = 'password_reset_access_token'

FOR_USER = 'for_user'
ONE_TIME_PERMISSION = 'one_time_permission'

REFRESH_TOKEN_COOKIE = 'refresh_token'



from rest_framework.permissions import BasePermission as DefaultBasePermission

from jwt_allauth.roles import STAFF_CODE, SUPER_USER_CODE


class BasePermission(DefaultBasePermission):
    """
    Custom base permission class for role-based access control using JWT claims.

    Extends DRF's BasePermission to check for roles in the JWT payload.
    **Automatically grants access to staff and superusers** in addition to specified roles.

    Behavior:

        - Checks JWT payload for 'role' claim
        - Allows access if role is in accepted_roles, STAFF_CODE, or SUPER_USER_CODE
        - Requires request.auth to contain decoded JWT payload
        - Staff and superusers (STAFF_CODE/SUPER_USER_CODE) always have access

    Class Attributes:
        accepted_roles (list): Required list of role codes that are allowed access.
                               Must be initialized in subclasses.

    Raises:
        ValueError: If accepted_roles is not properly initialized as a list
    """
    accepted_roles = None

    def _check_role_permission(self, request, include_staff=True):
        """
        Internal method to check role-based permissions.

        Args:
            request (Request): DRF request object containing JWT in auth attribute
            include_staff (bool): Whether to include staff and superuser roles in the check

        Returns:
            bool: True if authorized, False otherwise
        """
        if not isinstance(self.accepted_roles, list):
            raise ValueError('`accepted_roles` must be a list.')

        if not hasattr(request, 'auth'):
            return False

        if not request.auth or 'role' not in request.auth:
            return False

        roles_to_check = self.accepted_roles
        if include_staff:
            roles_to_check = self.accepted_roles + [STAFF_CODE, SUPER_USER_CODE]

        return request.auth['role'] in roles_to_check

    def has_permission(self, request, view):
        """
        Determine if the request should be permitted based on JWT roles.

        Args:
            request (Request): DRF request object containing JWT in auth attribute
            view (View): DRF view being accessed

        Returns:
            bool: True if authorized, False otherwise
        """
        return self._check_role_permission(request, include_staff=True)


class BasePermissionStaffExcluded(BasePermission):
    """
    Custom base permission class for role-based access control using JWT claims.

    Extends DRF's BasePermission to check for roles in the JWT payload.

    Behavior:

        - Checks JWT payload for 'role' claim
        - Allows access if role is in accepted_roles, STAFF_CODE, or SUPER_USER_CODE
        - Requires request.auth to contain decoded JWT payload

    Class Attributes:
        accepted_roles (list): Required list of role codes that are allowed access.
                               Must be initialized in subclasses.

    Raises:
        ValueError: If accepted_roles is not properly initialized as a list
    """
    accepted_roles = None

    def has_permission(self, request, view):
        """
        Determine if the request should be permitted based on JWT roles.

        Args:
            request (Request): DRF request object containing JWT in auth attribute
            view (View): DRF view being accessed

        Returns:
            bool: True if authorized, False otherwise

        Raises:
            ValueError: If accepted_roles is not a list
        """
        return self._check_role_permission(request, include_staff=False)



from django.contrib.auth.models import AbstractUser, Group, Permission
from django.contrib.auth.models import UserManager as DefaultUserManager
from django.db import models
from django.db.models import Q

from jwt_allauth.roles import STAFF_CODE, SUPER_USER_CODE


class UserManager(DefaultUserManager):
    def create_superuser(self, username, email=None, password=None, **extra_fields):
        extra_fields.setdefault("role", STAFF_CODE)
        if extra_fields.get("role") != STAFF_CODE:
            raise ValueError(f"Staff must have role={STAFF_CODE}.")
        return super().create_superuser(username, email=email, password=password, **extra_fields)

    def create_user(self, username, email=None, password=None, **extra_fields):
        if extra_fields.get('is_staff', False) is True:
            extra_fields.setdefault("role", STAFF_CODE)
        elif extra_fields.get('is_superuser', False) is True:
            extra_fields.setdefault("role", SUPER_USER_CODE)
        return super().create_user(username, email=email, password=password, **extra_fields)


class JAUser(AbstractUser):
    objects = UserManager()

    role = models.PositiveSmallIntegerField(null=False, default=0)
    groups = models.ManyToManyField(
        Group,
        related_name="custom_users",
        related_query_name="custom_user",
        blank=True,
    )
    user_permissions = models.ManyToManyField(
        Permission,
        related_name="custom_users",
        related_query_name="custom_user",
        blank=True,
    )

    class Meta:
        constraints = [
            models.CheckConstraint(
                check=~Q(is_staff=True) | Q(role=STAFF_CODE),
                name=f"staff_role_equal_to_{STAFF_CODE}"
            ),
            models.CheckConstraint(
                check=~(~Q(is_staff=True) & Q(is_superuser=True)) | Q(role=SUPER_USER_CODE),
                name=f"superuser_role_equal_to_{SUPER_USER_CODE}"
            ),
        ]



import re

from allauth.account import app_settings as allauth_settings
from allauth.account.adapter import get_adapter
from allauth.account.admin import EmailAddress
from allauth.account.utils import setup_user_email
# from allauth.socialaccount.helpers import complete_social_login
# from allauth.socialaccount.models import SocialAccount
# from allauth.socialaccount.providers.base import AuthProcess
from allauth.utils import get_username_max_length
from django.conf import settings as django_settings
# from django.contrib.auth import get_user_model
from django.db import transaction
# from django.http import HttpRequest
from django.utils.crypto import constant_time_compare
from django.utils.translation import gettext_lazy as _
# from requests.exceptions import HTTPError
from rest_framework import serializers


class RegisterSerializer(serializers.Serializer):
    username = serializers.CharField(
        max_length=get_username_max_length(),
        min_length=allauth_settings.USERNAME_MIN_LENGTH,
        required=False
    )
    email = serializers.EmailField(required=True, max_length=100)
    password1 = serializers.CharField(write_only=True)
    password2 = serializers.CharField(write_only=True)
    first_name = serializers.CharField(required=True, write_only=True, max_length=100)
    last_name = serializers.CharField(required=True, write_only=True, max_length=100)

    _has_phone_field = False

    def validate_username(self, username):
        username = get_adapter().clean_username(username)
        return username

    def validate_email(self, email):
        email = get_adapter().clean_email(email)
        if allauth_settings.UNIQUE_EMAIL:
            if EmailAddress.objects.filter(email=email, verified=True).exists():
                raise serializers.ValidationError(
                    _("A user is already registered with this e-mail address."))
            # delete previous non-verified registration attempts
            EmailAddress.objects.filter(email=email, verified=False).delete()
        return email

    def validate_password1(self, password):
        return get_adapter().clean_password(password)

    def validate_first_name(self, first_name):
        pattern = r'^[A-Za-zÀ-ÖØ-öø-ÿ ]+$'
        if not re.match(pattern, first_name):
            raise serializers.ValidationError('Incorrect format')
        first_name = re.sub(' +', ' ', first_name)
        return " ".join([txt.capitalize() for txt in first_name.split(" ")])

    def validate_last_name(self, last_name):
        pattern = r'^[A-Za-zÀ-ÖØ-öø-ÿ ]+$'
        if not re.match(pattern, last_name):
            raise serializers.ValidationError('Incorrect format')
        last_name = re.sub(' +', ' ', last_name)
        return " ".join([txt.capitalize() for txt in last_name.split(" ")])

    def validate(self, data):
        if not constant_time_compare(data['password1'], data['password2']):
            raise serializers.ValidationError(_("The two password fields didn't match."))
        return data

    def custom_signup(self, request, user):
        pass

    def get_cleaned_data(self):
        return {
            'username': self.validated_data.get('username', ''),
            'password1': self.validated_data.get('password1', ''),
            'email': self.validated_data.get('email', ''),
            'first_name': self.validated_data.get('first_name', ''),
            'last_name': self.validated_data.get('last_name', ''),
        }

    @transaction.atomic
    def save(self, request):
        adapter = get_adapter()
        user = adapter.new_user(request)
        self.cleaned_data = self.get_cleaned_data()
        adapter.save_user(request, user, self, commit=False)
        self.custom_signup(request, user)
        user.save()
        setup_user_email(request, user, [])
        if not bool(django_settings.EMAIL_VERIFICATION):
            email = EmailAddress.objects.filter(user=user.id).first()
            if email is not None:
                adapter.confirm_email(request, email)
        return user

#
# class SocialAccountSerializer(serializers.ModelSerializer):
#     """
#     serialize allauth SocialAccounts for use with a REST API
#     """
#     class Meta:
#         model = SocialAccount
#         fields = (
#             'id',
#             'provider',
#             'uid',
#             'last_login',
#             'date_joined',
#         )
#
#
# class SocialLoginSerializer(serializers.Serializer):
#     access_token = serializers.CharField(required=False, allow_blank=True)
#     code = serializers.CharField(required=False, allow_blank=True)
#
#     def _get_request(self):
#         request = self.context.get('request')
#         if not isinstance(request, HttpRequest):
#             request = request._request
#         return request
#
#     def get_social_login(self, adapter, app, token, response):
#         """
#         :param adapter: allauth.socialaccount Adapter subclass.
#             Usually OAuthAdapter or Auth2Adapter
#         :param app: `allauth.socialaccount.SocialApp` instance
#         :param token: `allauth.socialaccount.SocialToken` instance
#         :param response: Provider's response for OAuth1. Not used in the
#         :returns: A populated instance of the
#             `allauth.socialaccount.SocialLoginView` instance
#         """
#         request = self._get_request()
#         social_login = adapter.complete_login(request, app, token, response=response)
#         social_login.token = token
#         return social_login
#
#     def validate(self, attrs):
#         view = self.context.get('view')
#         request = self._get_request()
#
#         if not view:
#             raise serializers.ValidationError(
#                 _("View is not defined, pass it as a context variable")
#             )
#
#         adapter_class = getattr(view, 'adapter_class', None)
#         if not adapter_class:
#             raise serializers.ValidationError(_("Define adapter_class in view"))
#
#         adapter = adapter_class(request)
#         app = adapter.get_provider().get_app(request)
#
#         # More info on code vs access_token
#         # http://stackoverflow.com/questions/8666316/facebook-oauth-2-0-code-and-token
#
#         # Case 1: We received the access_token
#         if attrs.get('access_token'):
#             access_token = attrs.get('access_token')
#
#         # Case 2: We received the authorization code
#         elif attrs.get('code'):
#             self.callback_url = getattr(view, 'callback_url', None)
#             self.client_class = getattr(view, 'client_class', None)
#
#             if not self.callback_url:
#                 raise serializers.ValidationError(
#                     _("Define callback_url in view")
#                 )
#             if not self.client_class:
#                 raise serializers.ValidationError(
#                     _("Define client_class in view")
#                 )
#
#             code = attrs.get('code')
#
#             provider = adapter.get_provider()
#             scope = provider.get_scope(request)
#             client = self.client_class(
#                 request,
#                 app.client_id,
#                 app.secret,
#                 adapter.access_token_method,
#                 adapter.access_token_url,
#                 self.callback_url,
#                 scope
#             )
#             token = client.get_access_token(code)
#             access_token = token['access_token']
#
#         else:
#             raise serializers.ValidationError(
#                 _("Incorrect input. access_token or code is required."))
#
#         social_token = adapter.parse_token({'access_token': access_token})
#         social_token.app = app
#
#         try:
#             login = self.get_social_login(adapter, app, social_token, access_token)
#             complete_social_login(request, login)
#         except HTTPError:
#             raise serializers.ValidationError(_("Incorrect value"))
#
#         if not login.is_existing:
#             # We have an account already signed up in a different flow
#             # with the same email address: raise an exception.
#             # This needs to be handled in the frontend. We can not just
#             # link up the accounts due to security constraints
#             if allauth_settings.UNIQUE_EMAIL:
#                 # Do we have an account already with this email address?
#                 account_exists = get_user_model().objects.filter(
#                     email=login.user.email,
#                 ).exists()
#                 if account_exists:
#                     raise serializers.ValidationError(
#                         _("User is already registered with this e-mail address.")
#                     )
#
#             login.lookup()
#             login.save(request, connect=True)
#
#         attrs['user'] = login.account.user
#
#         return attrs
#
#
# class SocialConnectMixin(object):
#     def get_social_login(self, *args, **kwargs):
#         """
#         Set the social login process state to connect rather than login
#         Refer to the implementation of get_social_login in base class and to the
#         allauth.socialaccount.helpers module complete_social_login function.
#         """
#         social_login = super(SocialConnectMixin, self).get_social_login(*args, **kwargs)
#         social_login.state['process'] = AuthProcess.CONNECT
#         return social_login
#
#
# class SocialConnectSerializer(SocialConnectMixin, SocialLoginSerializer):
#     pass



from rest_framework_simplejwt.exceptions import AuthenticationFailed

from django.utils.translation import gettext_lazy as _
from rest_framework import status


class NotVerifiedEmail(AuthenticationFailed):
    status_code = status.HTTP_401_UNAUTHORIZED
    default_detail = _("User email is not verified")
    default_code = "email_not_verified"


class IncorrectCredentials(AuthenticationFailed):
    status_code = status.HTTP_401_UNAUTHORIZED
    default_detail = _("Incorrect credentials")
    default_code = "incorrect_credentials"



from typing import Dict, Any

from django.db.models import Q
from django.utils.crypto import constant_time_compare
from rest_framework import serializers
from rest_framework_simplejwt.exceptions import InvalidToken

from jwt_allauth.tokens.app_settings import RefreshToken
from jwt_allauth.tokens.models import RefreshTokenWhitelistModel


class RemoveRefreshTokenSerializer(serializers.Serializer):
    refresh = serializers.CharField()
    user = serializers.CurrentUserDefault()

    def validate(self, attrs: Dict[str, Any]) -> Dict[str, str]:
        refresh = RefreshToken(attrs["refresh"])  # The token is verified
        user_id = self.context.get('user')
        if user_id is None or 'session' not in refresh.payload:
            raise InvalidToken()
        if not constant_time_compare(user_id, refresh.payload['user_id']):
            raise InvalidToken()
        query = RefreshTokenWhitelistModel.objects.filter(
            Q(jti=refresh.payload["jti"]) | Q(session=refresh.payload["session"])
        )
        if not query.count() > 0:
            raise InvalidToken()
        query.delete()
        return {}



from django.conf import settings

from jwt_allauth.login.serializers import LoginSerializer as DefaultLoginSerializer
from jwt_allauth.password_change.serializers import PasswordChangeSerializer as DefaultPasswordChangeSerializer
from jwt_allauth.password_reset.serializers import PasswordResetSerializer as DefaultPasswordResetSerializer
from jwt_allauth.registration.serializers import RegisterSerializer as DefaultRegisterSerializer
from jwt_allauth.user_details.serializers import UserDetailsSerializer as DefaultUserDetailsSerializer
from jwt_allauth.utils import import_callable

serializers = getattr(settings, 'JWT_ALLAUTH_SERIALIZERS', {})

UserDetailsSerializer = import_callable(serializers.get('USER_DETAILS_SERIALIZER', DefaultUserDetailsSerializer))

LoginSerializer = import_callable(serializers.get('LOGIN_SERIALIZER', DefaultLoginSerializer))

PasswordResetSerializer = import_callable(serializers.get('PASSWORD_RESET_SERIALIZER', DefaultPasswordResetSerializer))

PasswordChangeSerializer = import_callable(
    serializers.get('PASSWORD_CHANGE_SERIALIZER', DefaultPasswordChangeSerializer)
)

RegisterSerializer = import_callable(serializers.get('REGISTER_SERIALIZER', DefaultRegisterSerializer))



from allauth.account.views import ConfirmEmailView
from django.http import HttpResponseRedirect, HttpResponseNotAllowed
from django.urls import reverse
from django.conf import settings
from rest_framework.permissions import AllowAny
from rest_framework.views import APIView

from jwt_allauth.constants import EMAIL_VERIFIED_REDIRECT
from jwt_allauth.tokens.models import RefreshTokenWhitelistModel
from jwt_allauth.registration.email_verification.serializers import VerifyEmailSerializer


class VerifyEmailView(APIView, ConfirmEmailView):
    permission_classes = (AllowAny,)
    allowed_methods = ('GET',)

    @staticmethod
    def get_serializer(*args, **kwargs):
        return VerifyEmailSerializer(*args, **kwargs)

    def get(self, request, *args, **kwargs):
        confirmation = self.get_object()

        # Enable refresh token
        refresh = RefreshTokenWhitelistModel.objects.filter(user=confirmation.email_address.user).first()
        if refresh:
            refresh.enabled = True
            refresh.save()

        confirmation.confirm(self.request)
        return HttpResponseRedirect(getattr(settings, EMAIL_VERIFIED_REDIRECT, reverse('jwt_allauth_email_verified')))

    def post(self, request, *args, **kwargs):
        return HttpResponseNotAllowed(['GET'])



from rest_framework.permissions import BasePermission as DefaultBasePermission
from rest_framework_simplejwt.exceptions import TokenError

from jwt_allauth.constants import PASS_RESET_ACCESS, PASS_RESET_COOKIE, FOR_USER, ONE_TIME_PERMISSION
from jwt_allauth.password_reset.models import SetPasswordTokenUser
from jwt_allauth.tokens.app_settings import RefreshToken


class ResetPasswordPermission(DefaultBasePermission):

    def has_permission(self, request, view):
        if bool(request.user and request.user.is_authenticated):
            # the user is authenticated: unexpected behaviour
            return False
        if hasattr(request, 'COOKIES') and PASS_RESET_COOKIE in request.COOKIES:
            access_token = request.COOKIES.get(PASS_RESET_COOKIE)
            try:
                access_token = RefreshToken.access_token_class(access_token)
            except TokenError:
                return False
            if access_token and ONE_TIME_PERMISSION in access_token and FOR_USER in access_token:
                if access_token[ONE_TIME_PERMISSION] == PASS_RESET_ACCESS:
                    request.user = SetPasswordTokenUser(access_token)
                    request.auth = access_token
                    return True
        return False



from allauth.account.adapter import get_adapter
from allauth.account.models import EmailAddress
from django.conf import settings
from django.contrib.auth.forms import PasswordResetForm
from django.contrib.sites.requests import RequestSite
from rest_framework import serializers

from jwt_allauth.constants import PASS_RESET
from jwt_allauth.password_change.serializers import PasswordChangeSerializer
from jwt_allauth.tokens.tokens import GenericToken
from jwt_allauth.utils import get_template_path


class PasswordResetSerializer(serializers.Serializer):
    """
    Serializer for requesting a password reset e-mail.
    """
    email = serializers.EmailField()

    password_reset_form_class = PasswordResetForm

    def get_email_options(self):
        """Override this method to change default e-mail options"""
        return {}

    def validate_email(self, value):
        value = get_adapter().clean_email(value)
        # Create PasswordResetForm with the serializer
        self.reset_form = self.password_reset_form_class(data=self.initial_data)
        if not self.reset_form.is_valid():
            raise serializers.ValidationError(self.reset_form.errors)

        return value

    def save(self):
        request = self.context.get('request')

        opts = {
            'use_https': request.is_secure(),
            'from_email': getattr(settings, 'DEFAULT_FROM_EMAIL'),
            'request': request,
            'domain_override': RequestSite(request).domain,
            'token_generator': GenericToken(request=request, purpose=PASS_RESET),
            'subject_template_name': get_template_path('PASS_RESET_SUBJECT', 'email/password/reset_email_subject.txt'),
            'html_email_template_name': get_template_path('PASS_RESET_EMAIL', 'email/password/reset_email_message.html'),
        }

        opts.update(self.get_email_options())
        # Check if the email is verified
        if EmailAddress.objects.filter(email=self.validated_data['email'], verified=True).count() > 0:
            self.reset_form.save(**opts)

class SetPasswordSerializer(PasswordChangeSerializer):
    old_password = None

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.old_password_field_enabled = False
        self.logout_on_password_change = False

        if 'old_password' in self.fields:
            self.fields.pop('old_password')

    def validate_old_password(self, value):
        pass



from django.contrib.auth import get_user_model
from rest_framework.generics import RetrieveUpdateAPIView
from rest_framework.permissions import IsAuthenticated

from jwt_allauth.app_settings import UserDetailsSerializer


class UserDetailsView(RetrieveUpdateAPIView):
    """
    Reads and updates UserModel fields
    Accepts GET, PATCH methods.

    Default accepted fields: username, first_name, last_name
    Default display fields: pk, username, email, first_name, last_name
    Read-only fields: pk, email

    Returns UserModel fields.
    """
    serializer_class = UserDetailsSerializer
    permission_classes = (IsAuthenticated,)
    http_method_names = ['get', 'patch', 'head', 'options']

    def get_object(self):
        return get_user_model().objects.get(id=self.request.user.id)

    def get_queryset(self):
        """
        Adding this method since it is sometimes called when using
        django-rest-swagger
        https://github.com/Tivix/django-rest-auth/issues/275
        """
        return get_user_model().objects.none()



from django.contrib.auth import get_user_model
from django.utils.translation import gettext_lazy as _
from rest_framework.generics import GenericAPIView
from rest_framework.permissions import IsAuthenticated
from rest_framework.response import Response
from rest_framework.throttling import UserRateThrottle

from jwt_allauth.app_settings import PasswordChangeSerializer
from jwt_allauth.utils import sensitive_post_parameters_m


class PasswordChangeView(GenericAPIView):
    """
    Calls Django Auth SetPasswordForm save method.

    Accepts the following POST parameters: new_password1, new_password2
    Returns the success/fail message.
    """
    serializer_class = PasswordChangeSerializer
    permission_classes = (IsAuthenticated,)
    throttle_classes = [UserRateThrottle]

    @sensitive_post_parameters_m
    def dispatch(self, *args, **kwargs):
        return super(PasswordChangeView, self).dispatch(*args, **kwargs)

    def post(self, request):
        # Load the user in the request
        request.user = get_user_model().objects.get(id=self.request.user.id)
        serializer = self.get_serializer(data=request.data)
        serializer.is_valid(raise_exception=True)
        serializer.save()
        return Response({"detail": _("New password has been saved.")})



from django.conf import settings

from jwt_allauth.tokens.tokens import RefreshToken as DefaultRefreshToken
from jwt_allauth.utils import import_callable

RefreshToken = import_callable(getattr(settings, 'JWT_ALLAUTH_REFRESH_TOKEN', DefaultRefreshToken))



from django.contrib.auth import get_user_model
from rest_framework import serializers


class UserDetailsSerializer(serializers.ModelSerializer):
    """
    User model w/o password
    """
    class Meta:
        model = get_user_model()
        fields = ('email', 'first_name', 'last_name')
        read_only_fields = ('email', )
